import requests

from ..repos import APIRepo

class FEAPIRepo(APIRepo):
    def __init__(self, url=None):
        super().__init__(url)

    def retrieve_pipelines(self, name, limit):
        url = self.url + "pipelines"
        if name is not None:
            url += "?match=" + name
        if limit is not None:
            if name is None:
                url += "?limit=" + str(limit)
            else:
                url += "&limit=" + str(limit)
        response = requests.get(url)
        return self.format_response(response)
    
    def retrieve_pipeline(self, name):
        response = requests.get(self.url + "pipelines?name=" + name)
        return self.format_response(response)
    
    def get_pipeline_by_id(self, pipeline_id):
        response = requests.get(self.url + "pipelines/" + pipeline_id)
        return self.format_response(response)
    
    def create_pipeline(self, metadata, user):
        response = requests.post(
            self.url + "pipelines",
            json=metadata,
            headers=self.generate_headers(user),
        )
        return self.format_response(response)

    def complete_ingestion(self, pipeline_id, version, size, user):
        response = requests.post(
            self.url + "pipelines/complete/" + pipeline_id,
            json={"version": version, "size": size},
            headers=self.generate_headers(user),
        )
        return self.format_response(response)
    
    def deactivate_pipeline(self, pipeline_id, user):
        response = requests.patch(
            self.url + "pipelines/deactivate/" + pipeline_id,
            headers=self.generate_headers(user),
        )
        return self.format_response(response)



from pathlib import Path

from ..repos import ModelsAPIRepo
from ..files.ingest import prep_ingest_stac, prep_ingest_folder, ingest, ingest_virtual, ingest_catalog

def retrieve_model(metadata, user, private=False):
	repo = ModelsAPIRepo()
	data, error = repo.retrieve_model(metadata.name)
	# print(data, error)
	if data and data["uid"] != user["uid"]:
		raise Exception("Model already exists.")
	if error and error == "Model doesn't exist":
		# create model
		data, error = repo.create_model(metadata.dict(), user)
		if error:
			raise Exception(error)
	return data

def ingest_model(
	path,
	verbose=False,
	logger=print,
	force_metadata_update=False,
	sync_metadata=False,
	private=False,
):
	path = Path(path)
	if not path.is_dir():
		raise Exception("Path must be a folder")
	if "catalog.json" in [f.name for f in path.iterdir()]:
		prep_ingest_stac(path, logger)
	else:
		prep_ingest_folder(path, verbose, logger, force_metadata_update, sync_metadata)
	return ingest(path, ModelsAPIRepo(), retrieve_model, 'models', private)

def ingest_virtual_model( # could work for a list of paths with minimal changes...
	path,
	links,
	metadata = None, 
	logger=print,
	user=None,
):
	return ingest_virtual(path, links, ModelsAPIRepo(), retrieve_model, 'models', metadata, logger)

def ingest_model_catalog(path, logger=None):
	path = Path(path)
	return ingest_catalog(path, ModelsAPIRepo(), retrieve_model, 'models')


import requests

from ..repos import APIRepo

class ModelsAPIRepo(APIRepo):
    def __init__(self, url=None):
        super().__init__(url)

    def retrieve_models(self, name, limit):
        url = self.url + "models"
        if name is not None:
            url += "?match=" + name
        if limit is not None:
            if name is None:
                url += "?limit=" + str(limit)
            else:
                url += "&limit=" + str(limit)
        response = requests.get(url)
        return self.format_response(response)

    def retrieve_model(self, name):
        response = requests.get(self.url + "models?name=" + name)
        return self.format_response(response)
    
    def create_model(self, metadata, user):
        response = requests.post(
            self.url + "models",
            json=metadata,
            headers=self.generate_headers(user),
        )
        return self.format_response(response)

    def complete_ingestion(self, model_id, version, size, user):
        response = requests.post(
            self.url + "models/complete/" + model_id,
            json={"version": version, "size": size},
            headers=self.generate_headers(user),
        )
        return self.format_response(response)

    def deactivate_model(self, model_name, user):
        response = requests.patch(
            self.url + "models/deactivate/" + model_name,
            headers=self.generate_headers(user),
        )
        return self.format_response(response)
